﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [StorableClass]
  [Item("ScatterPlotAnalyzer", "Creates a Scatterplot for the current and the best known front (see Multi-Objective Performance Metrics - Shodhganga for more information)")]
  public class ScatterPlotAnalyzer : MOTFAnalyzer {

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
    }
    [StorableConstructor]
    protected ScatterPlotAnalyzer(bool deserializing) : base(deserializing) { }
    public ScatterPlotAnalyzer(ScatterPlotAnalyzer original, Cloner cloner) : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ScatterPlotAnalyzer(this, cloner);
    }


    public ScatterPlotAnalyzer() { }






    public override void Analyze(Individual[] individuals, double[][] qualities, ResultCollection results) {
      if (qualities == null || qualities.Length < 1) return;
      int objectives = qualities[0].Length;
      double[][] opf = new double[0][];
      var optmialFront = TestFunctionParameter.ActualValue.OptimalParetoFront(objectives);
      if (optmialFront != null) {
        opf = optmialFront.Select(s => s.ToArray()).ToArray();
      }


      var qualityClones = qualities.Select(s => s.ToArray()).ToArray(); //fast Clone of jagged arrays
      var solutionClones = individuals.Select(s => s.RealVector().ToArray()).ToArray();

      if (!results.ContainsKey("Scatterplot")) {
        results.Add(new Result("Scatterplot", typeof(IMOFrontModel)));

      } else {
        //((MOSolution)results["Scatterplot"].Value).Qualities = qualityClones;
        //((MOSolution)results["Scatterplot"].Value).Solutions = solutionClones;

        //These two only when the TestFunction changed??
        //((MOSolution)results["Scatterplot"].Value).ParetoFront = opf;
        //((MOSolution)results["Scatterplot"].Value).Objectives = objectives;
      }

      results["Scatterplot"].Value = new MOSolution(qualityClones, solutionClones, opf, objectives);
    }
  }
}
